/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include <InterViews/filechooser.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/catalog.h>
#include <Unidraw/iterator.h>
#include <Unidraw/editor.h>
#include <Unidraw/statevars.h>
#include <Unidraw/dialogs.h>

#include "dctclasses.h"
#include "dctcat.h"
#include "dcted.h"
#include "dctfile.h"

#include <stream.h>
#include <string.h>

extern "C" {
#include "dbStaticLib.h"
};

//------------------------commands for save,saveas,open,new---------------

static void UpdateCompNameVars()
{
    Iterator i;
 
    for (unidraw->First(i); !unidraw->Done(i); unidraw->Next(i))
	{
        Editor* ed = unidraw->GetEditor(i);
        CompNameVar* compNameVar = (CompNameVar*) ed->GetState("CompNameVar");
        if (compNameVar != nil) compNameVar->UpdateName();
    }
}

static boolean Writable (Component* comp)
{
	Catalog* catalog = unidraw->GetCatalog();
	const char* name = catalog->GetName(comp);
	return name == nil || (catalog->Exists(name) && catalog->Writable(name));
}

static boolean ReadyToClose (Editor* ed)
{
    ModifStatusVar* mv = (ModifStatusVar*) ed->GetState("ModifStatusVar");
 
    if (mv != nil && Writable(mv->GetComponent()) && mv->GetModifStatus())
	{
        ConfirmDialog dialog("Save changes?");
 
        ed->InsertDialog(&dialog);
        char resp = dialog.Confirm();
        ed->RemoveDialog(&dialog);
 
        if (resp == '\007')
            return false;                       // confirm dialog aborted
		else if (resp == 'y')
		{
            DctSaveCompCmd saveComp(ed);
            saveComp.Execute();
 
            if (mv->GetModifStatus())
                return false;                   // save dialog was aborted
        }
    }
    return true;
}

//---new component cmd---

DctNewCompCmd::DctNewCompCmd(ControlInfo* c, Component* p):Command(c)
{ prototype_=p; }
DctNewCompCmd::DctNewCompCmd(Editor* e, Component* p):Command(e)
{ prototype_=p; }

DctNewCompCmd::~DctNewCompCmd () { delete prototype_; }
boolean DctNewCompCmd::Reversible () { return false; }
ClassId DctNewCompCmd::GetClassId() { return DCTNEWCOMP_CMD; }
boolean DctNewCompCmd::IsA(ClassId id)
{ return DCTNEWCOMP_CMD == id || Command::IsA(id); }

void DctNewCompCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	Dct* ed = (Dct*)GetEditor();
	Component* orig = ed->GetComponent();
	Component* comp = prototype_->Copy();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	CompNameVar* compNameVar = (CompNameVar*) ed->GetState("CompNameVar");
    ModifStatusVar* modifVar = (ModifStatusVar*)ed->GetState("ModifStatusVar");

	if(!ReadyToClose(ed)) return;

    if (compNameVar != nil) compNameVar->SetComponent(comp);
    if (modifVar != nil) modifVar->SetComponent(comp);
 
    ed->SetComponent(comp);
    ed->Update();
 
    if (orig != nil && unidraw->FindAny(orig) == nil)
	{
        Component* root = orig->GetRoot();
        delete root;
    }

	dbFreeRecords(db);
	nv->SetName(du->GetName());
}

Command* DctNewCompCmd::Copy()
{
	Command* copy = new DctNewCompCmd(CopyControlInfo(), prototype_->Copy());
	InitCopy(copy);
	return copy;
}

void DctNewCompCmd::Read(istream& in)
{ Command::Read(in); prototype_ = unidraw->GetCatalog()->ReadComponent(in); }
void DctNewCompCmd::Write(ostream& out)
{ Command::Write(out); unidraw->GetCatalog()->WriteComponent(prototype_, out); }

//---save component cmd---
 
boolean DctSaveCompCmd::IsA(ClassId id)
{ return SAVECOMP_CMD == id || Command::IsA(id); }
ClassId DctSaveCompCmd::GetClassId () { return SAVECOMP_CMD; }
boolean DctSaveCompCmd::Reversible () { return false; }
DctSaveCompCmd::DctSaveCompCmd (ControlInfo* c) : Command(c) { }
DctSaveCompCmd::DctSaveCompCmd (Editor* ed) : Command(ed) { }
 
Command* DctSaveCompCmd::Copy()
{
    Command* copy = new DctSaveCompCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void DctSaveCompCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	Dct* ed = (Dct*)GetEditor();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	ModifStatusVar* modifVar = (ModifStatusVar*)ed->GetState("ModifStatusVar");
    CompNameVar* compNameVar = (CompNameVar*) ed->GetState("CompNameVar");
    const char* name = (compNameVar == nil) ? nil : compNameVar->GetName();

	if (name == nil)
	{
        DctSaveCompAsCmd saveCompAs(ed);
        saveCompAs.Execute();
	}
	else if(modifVar == nil || modifVar->GetModifStatus())
	{
		DctCatalog* catalog = (DctCatalog*)unidraw->GetCatalog();
		Component* comp;

		if((catalog->Retrieve(name, comp) && catalog->Save(comp,name,"scum")))
		{
			if (modifVar != nil) modifVar->SetModifStatus(false);
			unidraw->ClearHistory(comp);
		}
		else
		{
            // char title[CHARBUFSIZE];
            // const char* reason=!Writable(comp) ? "(File not writable.)" : "";
            // sprintf(title, "Couldn't save! %s", reason);
            // char subtitle[CHARBUFSIZE];
            // const char* domain=unidraw->GetCatalog()->GetAttribute("domain");
            // domain = (domain == nil) ? "component" : domain;
            // sprintf(subtitle, "Save this %s as:", domain);
 
            DctSaveCompAsCmd saveCompAs(ed);
            saveCompAs.Execute();
		}
	}
}

//---save as component cmd---

ClassId DctSaveCompAsCmd::GetClassId() { return DCTSAVECOMPAS_CMD; }
boolean DctSaveCompAsCmd::IsA(ClassId id)
 { return DCTSAVECOMPAS_CMD == id || Command::IsA(id); }
 
DctSaveCompAsCmd::DctSaveCompAsCmd(ControlInfo* c):Command(c) { }
DctSaveCompAsCmd::DctSaveCompAsCmd(Editor* ed):Command(ed) { }
DctSaveCompAsCmd::~DctSaveCompAsCmd () { }
 
Command* DctSaveCompAsCmd::Copy()
{
    Command* copy = new DctSaveCompAsCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

boolean DctSaveCompAsCmd::Reversible () { return false; }

void DctSaveCompAsCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	Dct* ed = (Dct*)GetEditor();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	ModifStatusVar* modifVar = (ModifStatusVar*)ed->GetState("ModifStatusVar");
	DctCatalog* catalog = (DctCatalog*)unidraw->GetCatalog();
	boolean complete = false;
	char buf[100];
	char *name,*db_name,*p;
	CompNameVar* cnv;
	Component* comp;
	FILE* fd;

	// FileChooser fc("file_chew","Save As","",".",10,24,"Save",Center);
	FileDialog fc("File Chew","Save As");

    ed->InsertDialog(&fc);
	global_dialog_wn_delete(&fc);

	while(complete==false)
	{
		switch(fc.GetSaveName(name))
		{
		case FDIAL_CANCEL: complete=true; break;
		case FDIAL_DBD:
			if((fd=fopen(name,"w")))
			{
				dbWriteMenuFP(db,fd,NULL);
				dbWriteRecordTypeFP(db,fd,NULL);
				dbWriteDeviceFP(db,fd);
				dbWriteBreaktableFP(db,fd);
				fclose(fd);
			}
			else
			{
				AcknowledgeDialog ack("Could not write in the dbd file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			complete=true;
			break;
		case FDIAL_DB:
			if(dbWriteRecord(db,name,NULL,1))
			{
				AcknowledgeDialog ack("Could not write in the db file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			complete=true;
			break;
		case FDIAL_NORMAL:
			if(catalog->Exists(name) && catalog->Writable(name))
			{
				// file exists already
				ConfirmDialog dialog(name,"File exists, overwrite it?");
				ed->InsertDialog(&dialog);
				char confirmation = dialog.Confirm();
				ed->RemoveDialog(&dialog);
				if (confirmation != 'y') complete=true;
			}
			if(complete==false)
			{
				cnv=(CompNameVar*)ed->GetState("CompNameVar");
				const char* oldname=(cnv==nil)?nil:cnv->GetName();
				comp=ed->GetComponent();

				if(catalog->Exists(name) && !catalog->Writable(name))
				{
					fc.SetSubtitle("File selected is not writable!");
					fc.Update();
				}
				else
				{
					if(oldname==nil)
						comp=comp->GetRoot();
					else
					{
						catalog->Retrieve(oldname,comp);
						catalog->Forget(comp);
					}
	
					modifVar=
						(ModifStatusVar*)ed->GetState("ModifStatusVar");

					if(catalog->Save(comp,name,"scum"))
					{
						if(modifVar!=nil)
							modifVar->SetModifStatus(false);

						unidraw->ClearHistory(comp);
						UpdateCompNameVars();
						du->SetName(name);
						nv->SetName(name);
						complete=true;
					}
					else
					{
						if(modifVar!=nil) modifVar->Notify();
						UpdateCompNameVars();
						fc.SetSubtitle("File could not be saved!");
						fc.Update();
					}
				}
			}
			break;
		default: break; // ??????????????
		}
		if(name) delete [] name;
	}

	ed->RemoveDialog(&fc);
}

//---view component cmd---

DctViewCompCmd::DctViewCompCmd(ControlInfo* c) : Command(c) { }
DctViewCompCmd::DctViewCompCmd(Editor* ed) : Command(ed){ }
DctViewCompCmd::~DctViewCompCmd () { }
boolean DctViewCompCmd::Reversible () { return false; }
ClassId DctViewCompCmd::GetClassId() { return DCTVIEWCOMP_CMD; }
boolean DctViewCompCmd::IsA(ClassId id)
{ return DCTVIEWCOMP_CMD == id || Command::IsA(id); }

Command* DctViewCompCmd::Copy ()
{
    Command* copy = new DctViewCompCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void DctViewCompCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	Dct* ed = (Dct*)GetEditor();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	ModifStatusVar* modif = (ModifStatusVar*) ed->GetState("ModifStatusVar");
	CompNameVar* cnv = (CompNameVar*)ed->GetState("CompNameVar");

	if( !ReadyToClose(ed) ) return;

	boolean complete = false;
	// FileChooser fc("file_chew","Open","",".",10,24,"Open",Center);
	FileDialog fc("File Chew","Open");
	DctCatalog* catalog = (DctCatalog*)du->GetCatalog();
	GraphicComp* comp;
	char* name;

	ed->InsertDialog(&fc);
	global_dialog_wn_delete(&fc);

	while(complete==false)
	{
		switch(fc.GetOpenName(name))
		{
		case FDIAL_CANCEL:
			complete=true;
			break;
		case FDIAL_DBD:
			if(du->AddDBD(name))
			{
				AcknowledgeDialog ack("Could not read in the dbd file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			else
			{
				if(modif != nil) modif->SetModifStatus(true);
			}
			complete=true;
			break;
		case FDIAL_DB:
			if(dbReadDatabase(&db,name,NULL,NULL))
			{
				AcknowledgeDialog ack("Could not read in the db file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			else
			{
				if(modif != nil) modif->SetModifStatus(true);
			}
			complete=true;
			break;
		case FDIAL_NORMAL:
			// the user hit ok button
			du->MakeCatDatabase();
			
			if(catalog->Retrieve("you_scum",name, (Component*&) comp))
			{
				Component* orig = ed->GetComponent();
				ed->SetComponent(comp);
				unidraw->Update();

				if(cnv != nil) cnv->SetComponent(comp);
				if(modif != nil) modif->SetComponent(comp);

				if(orig != nil && unidraw->FindAny(orig) == nil)
				{
					Component* root = orig->GetRoot();
					delete root;
				}

				du->UseCatDatabase();
				DBBASE* dbx = du->GetDatabase();
				ed->SetContainer(dbx);
				nv->SetName(du->GetName());
				complete=true;
			}
			else
			{
				fc.SetSubtitle("Could not read the file!");
				fc.Update();
			}
			break;
		default: break; // ?????????????
		}
		if(name) delete [] name;
	}
	ed->RemoveDialog(&fc);
}

